/**
 * @file puzzle.x
 * Implementation of the inline functions of puzzle.h
 */
#ifndef _PUZZLE_X
#define _PUZZLE_X

#include <assert.h>

/**
 * @addtogroup Puzzle_Group Puzzle
 *
 * @section Puzzle_Sec_Inline Inline
 * See http://www.greenend.org.uk/rjk/2003/03/inline.html for inline functions in C
 *
 * @{
 */

/** 
 * @brief Get the value of a cell.
 *
 * @param p the puzzle
 * @param r row
 * @param c column
 * @return the value of the cell or 0 if the cell is empty
 */
static inline int puzzle_get_cell(const Puzzle *p, int r, int c)
{
	assert(p);
	assert(0 <= r && r < 9 && 0 <= c && c < 9);
	return p->g[r][c];
}

/** 
 * @brief Check if a value is a possible candidate.
 *
 * @param p the puzzle
 * @param r row
 * @param c column
 * @param v value
 * @return 1 if @p v is a candidate.<br/>
 * 	0 otherwise
 */
static inline int puzzle_get_candidate(const Puzzle *p, int r, int c, int v)
{
	assert(p);
	assert(0 <= r && r < 9 && 0 <= c && c < 9 && 1 <= v && v <= 9);
	return p->sg[r][c][v];
}

/**
 * @}
 */
#endif /* _PUZZLE_X */
