/**
 * @file solver.h
 * Interface of <b>Solver</b> module.
 */

#ifndef _SOLVER_H
#define _SOLVER_H

#include "puzzle.h"

/**
 * @addtogroup Solver_Group Solver
 *
 * The Solver module offers methods for solving Sudoku puzzles.<br/>
 * The following methods are implemented:
 * 	- finding naked singles: _naked_singles()
 * 	- finding hidden singles: _hidden_singles()
 * 	- brute force: _make_guess()
 *
 * @section Solver_Sec_Man Manual
 * Example:
 * @code
 * char s[] = "4.....8.5.3..........7......2.....6.....8.4......1.......6.3.7.5..2.....1.4......";
 * Puzzle p;
 * if(!puzzle_set_grid_str(&p, s))
 * 	return 1;
 * if(solver_solve(&p))
 * 	puzzle_print(&p);
 * else printf("The puzzle has no solution.\n");
 * @endcode
 *
 * @{
 */

/* Public */
int solver_solve(Puzzle *p);

/* Private */
int _naked_singles(Puzzle *p);
int _hidden_singles(Puzzle *p);
int _make_guess(const Puzzle *p, int guess[3]);

/**
 * @}
 */

#endif /* _SOLVER_H */
