/**
 * @file sudoku.c
 * Main application file.
 */

/**
 * @mainpage Sudoku
 *
 * @author Kayané Makhsiyan <kayane.makhsiyan@insa-lyon.fr>
 * @author Mihai Moraru <mihai.moraru@insa-lyon.fr>
 * @date november 2010
 *
 * @section Sudoku_Sec_Intro Introduction
 * Course: 3TC-AGP<br/>
 * Professor: Tanguy Risset<br/>
 * Assignment: TP2<br/>
 *
 * The objectives we sought for in this project encompass the purpose of the original subject (@ref Sudoku_Sec_Subject).<br/>
 * Throughout this project, we acquired or trained the following areas:
 * 	- how to properly document a project
 * 	- advanced use of Doxygen
 * 		- how to use a large variety of special commands (tags)
 * 		- customizing the Doxyfile
 * 		- customizing the output (doxygen.css)
 * 	- advanced use of Makefiles
 * 		- implicit rules
 * 		- pattern rules
 * 		- .PHONY targets
 * 		- recursion
 * 	- subtleties of the C programming language
 * 		- arrays vs. pointers<br/>
 * 		We used (int *) instead of the proposed int[9][9]. See puzzle_set_grid().
 * 		- inline functions<br/>
 * 		See @ref Puzzle_Sec_Inline and puzzle.x
 *
 * The rest of the project is pretty well documented. See:
 * 	- @ref Puzzle_Group
 * 	- @ref Solver_Group
 *
 * @section Sudoku_Sec_Man Manual
 * This sections discusses the use of the @c sudoku program.
 * @c sudoku reads Sudoku puzzles from the input files and gives a solution.<br/>
 * Usage: @code sudoku [input_file1] [input_file2] ... @endcode<br/>
 * If the first argument has the value '-' (dash), then @c sudoku reads the puzzles from stdin.<br/>
 * For each file, the program prints the solutions it finds and some statistics indicating how many puzzles were solved.<br/>
 * The program reads lines of 81 characters at a time, each line being a string representation of a Sudoku puzzle. See puzzle_str2grid().
 *
 * @section Sudoku_Sec_Test Testing
 * The program has been tested on 95 hard puzzles against the Python implementation of Peter Norvig (www.norvig.com).
 *
 * @section Sudoku_Sec_Subject Subject
 * Here follows the original subject
 * @image html TP2_1.png
 * @image html TP2_2.png
 * @image html TP2_3.png
 *
 */
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include "puzzle.h"
#include "solver.h"

void parse(FILE *f);

int main(int argc, char *argv[])
{
	if(argc == 1) {
		fprintf(stderr, "Usage: %s [input_file1] [input_file2] ...\n", argv[0]);
		fprintf(stderr, "You can also use '-' as the first argument to read sudokus from stdin.\n");
	}
	if(strcmp(argv[1], "-") == 0)
		parse(stdin);
	else for(int i = 1; i < argc; ++i) {
		if(!freopen(argv[i], "r", stdin)) {
			fprintf(stderr, "Error opening file '%s' for reading.\n", argv[i]);
			continue;
		}
		parse(stdin);
	}
	return 0;
}

void parse(FILE *f)
{
	assert(f);

	int read = 0, solved = 0;
	char s[82]; // 81 + '\0'
	Puzzle p;

	while(fgets(s, 82, f)) {
		if(strlen(s) < 81)
			continue;
		puzzle_clear(&p);
		if(!puzzle_set_grid_str(&p, s))
			continue;
		++read;
		solved += solver_solve(&p);
		puzzle_print(&p);
		printf("\n");
	}
	printf("## Got %d out of %d\n", solved, read);
}
